@@ -15,6 +15,7 @@ from typing import Iterator
 from typing import List
 from typing import Optional
 from typing import Tuple
+from typing import Union
 from typing import cast
 
 import humps
@@ -36,9 +37,14 @@ from dipdup.http import HTTPGateway
 from dipdup.models import Schema
 from dipdup.utils import iter_files
 from dipdup.utils import pascal_to_snake
+from dipdup.utils import snake_to_pascal
 from dipdup.utils.database import get_connection
 from dipdup.utils.database import iter_models
 
+RelationalFieldT = Union[
+    fields.relational.ForeignKeyFieldInstance,
+    fields.relational.ManyToManyFieldInstance,
+]
 _get_fields_query = '''
 query introspectionQuery($name: String!) {
   __type(name: $name) {
@@ -83,7 +89,6 @@ class Field:
 
 class HasuraGateway(HTTPGateway):
     _default_http_config = HTTPConfig(
-        cache=False,
         # NOTE: Does not apply to initial healthcheck
         retry_count=3,
         retry_sleep=1,
@@ -121,24 +126,36 @@ class HasuraGateway(HTTPGateway):
             self._logger.info('Metadata is up to date, no action required')
             return
 
-        # NOTE: Find chosen source and overwrite its tables
+        # NOTE: Find chosen source and overwrite its tables, create if missing and allowed.
         source_name = self._hasura_config.source
-        for source in metadata['sources']:
-            if source['name'] == source_name:
-                source['tables'] = await self._generate_source_tables_metadata()
-                break
-        else:
-            raise HasuraError(f'Source `{source_name}` not found in metadata')
+
+        if (source := self._get_source(metadata, source_name)) is None:
+            if not self._hasura_config.create_source:
+                raise HasuraError(f'Source `{source_name}` not found in metadata. Set `create_source` flag to create it.')
+
+            await self._create_source()
+            metadata = await self._fetch_metadata()
+            if (source := self._get_source(metadata, source_name)) is None:
+                raise HasuraError(f'Source `{source_name}` not found in metadata after creation.')
+
+        source['tables'] = await self._generate_source_tables_metadata()
+
+        # NOTE: Don't forget to invalidate old queries, customization will fail otherwise.
+        source['query_collections'] = []
+        source['rest_endpoints'] = []
+
         await self._replace_metadata(metadata)
 
         # NOTE: Apply table customizations before generating queries
-        await self._apply_table_customization()
+        await self._apply_table_customization(source)
         metadata = await self._fetch_metadata()
+        if (source := self._get_source(metadata, source_name)) is None:
+            raise HasuraError(f'Source `{source_name}` not found in metadata after table customization.')
 
         # NOTE: Generate and apply queries and REST endpoints
         query_collections_metadata = await self._generate_query_collections_metadata()
         self._logger.info('Adding %s generated and user-defined queries', len(query_collections_metadata))
-        metadata['query_collections'] = [
+        source['query_collections'] = [
             {
                 "name": "allowed-queries",
                 "definition": {"queries": query_collections_metadata},
@@ -149,11 +166,12 @@ class HasuraGateway(HTTPGateway):
             self._logger.info('Adding %s REST endpoints', len(query_collections_metadata))
             query_names = [q['name'] for q in query_collections_metadata]
             rest_endpoints_metadata = await self._generate_rest_endpoints_metadata(query_names)
-            metadata['rest_endpoints'] = rest_endpoints_metadata
+            source['rest_endpoints'] = rest_endpoints_metadata
 
         await self._replace_metadata(metadata)
 
-        # NOTE: Fetch metadata once again (to do: find out why is it necessary) and save its hash for future comparisons
+        # TODO: Find out why it is necessary
+        # NOTE: Fetch metadata once again and save its hash for future comparisons
         metadata = await self._fetch_metadata()
         metadata_hash = self._hash_metadata(metadata)
         hasura_schema.hash = metadata_hash
@@ -161,12 +179,18 @@ class HasuraGateway(HTTPGateway):
 
         self._logger.info('Hasura instance has been configured')
 
+    def _get_source(self, metadata: Dict[str, Any], name: str) -> Optional[Dict[str, Any]]:
+        for source in metadata['sources']:
+            if source['name'] == name:
+                return source
+        else:
+            return None
+
     async def _hasura_request(self, endpoint: str, json: Dict[str, Any]) -> Dict[str, Any]:
         self._logger.debug('Sending `%s` request: %s', endpoint, dump_json(json))
         try:
             result = await self.request(
                 method='post',
-                cache=False,
                 url=f'{self._hasura_config.url}/v1/{endpoint}',
                 json=json,
                 headers=self._hasura_config.headers,
@@ -203,13 +227,34 @@ class HasuraGateway(HTTPGateway):
 
         self._logger.info('Connected to Hasura %s', version)
 
+    async def _create_source(self) -> Dict[str, Any]:
+        self._logger.info(f'Adding source `{self._hasura_config.source}`')
+        return await self._hasura_request(
+            endpoint='metadata',
+            json={
+                'type': 'pg_add_source',
+                'args': {
+                    'name': self._hasura_config.source,
+                    'configuration': {
+                        'connection_info': {
+                            'database_url': {
+                                'connection_parameters': self._database_config.hasura_connection_parameters,
+                            },
+                            'use_prepared_statements': True,
+                        }
+                    },
+                    'replace_configuration': True,
+                },
+            },
+        )
+
     async def _fetch_metadata(self) -> Dict[str, Any]:
         self._logger.info('Fetching existing metadata')
         return await self._hasura_request(
             endpoint='metadata',
             json={
-                "type": "export_metadata",
-                "args": {},
+                'type': 'export_metadata',
+                'args': {},
             },
         )
 
@@ -278,7 +323,7 @@ class HasuraGateway(HTTPGateway):
                     metadata_tables[table_name]['object_relationships'].append(
                         self._format_object_relationship(
                             name=field_name,
-                            column=field_name + '_id',
+                            column=self._get_relation_source_field(field),
                         )
                     )
                     if field.related_name:
@@ -286,7 +331,7 @@ class HasuraGateway(HTTPGateway):
                             self._format_array_relationship(
                                 related_name=field.related_name,
                                 table=table_name,
-                                column=field_name + '_id',
+                                column=self._get_relation_source_field(field),
                             )
                         )
 
@@ -298,13 +343,13 @@ class HasuraGateway(HTTPGateway):
                     metadata_tables[junction_table_name]['object_relationships'].append(
                         self._format_object_relationship(
                             name=related_table_name,
-                            column=related_table_name + '_id',
+                            column=self._get_relation_source_field(field),
                         )
                     )
                     metadata_tables[junction_table_name]['object_relationships'].append(
                         self._format_object_relationship(
                             name=table_name,
-                            column=table_name + '_id',
+                            column=self._get_relation_source_field(field),
                         )
                     )
                     if field.related_name:
@@ -312,7 +357,7 @@ class HasuraGateway(HTTPGateway):
                             self._format_array_relationship(
                                 related_name=f'{related_table_name}_{field.related_name}',
                                 table=junction_table_name,
-                                column=related_table_name + '_id',
+                                column=self._get_relation_source_field(field),
                             )
                         )
 
@@ -334,7 +379,14 @@ class HasuraGateway(HTTPGateway):
                 raise RuntimeError(f'Table `{table_name}` has no primary key. How is that possible?')
 
             fields = await self._get_fields(table_name)
-            queries.append(self._format_rest_query(table_name, table_name, filter, fields))
+            queries.append(
+                self._format_rest_query(
+                    name=table_name,
+                    table=table_name,
+                    filter=filter,
+                    fields=fields,
+                )
+            )
 
         for query_name, query in self._iterate_graphql_queries():
             queries.append({'name': query_name, 'query': query})
@@ -400,17 +452,22 @@ class HasuraGateway(HTTPGateway):
 
         return fields
 
-    async def _apply_table_customization(self) -> None:
+    async def _apply_table_customization(self, source: Dict[str, Any]) -> None:
         """Convert table and column names to camelCase.
 
         Based on https://github.com/m-rgba/hasura-snake-to-camel
         """
 
+        # NOTE: Build a set of table names for our source
+        table_names = {t['table']['name'] for t in source['tables']}
         tables = await self._get_fields()
 
         # TODO: Bulk request
         for table in tables:
-            custom_root_fields = self._format_custom_root_fields(table)
+            if table.root not in table_names:
+                continue
+
+            custom_root_fields = self._format_custom_root_fields(table.root)
             columns = await self._get_fields(table.root)
             custom_column_names = self._format_custom_column_names(columns)
             args: Dict[str, Any] = {
@@ -479,9 +536,7 @@ class HasuraGateway(HTTPGateway):
             "comment": None,
         }
 
-    def _format_custom_root_fields(self, table: Field) -> Dict[str, Any]:
-        table_name = table.root
-
+    def _format_custom_root_fields(self, table_name: str) -> Dict[str, Any]:
         def _fmt(fmt: str) -> str:
             if self._hasura_config.camel_case:
                 return humps.camelize(fmt.format(table_name))
@@ -559,3 +614,13 @@ class HasuraGateway(HTTPGateway):
                 "limit": self._hasura_config.select_limit,
             },
         }
+
+    def _get_relation_source_field(self, field: RelationalFieldT) -> str:
+        if source_field := field.source_field:
+            name = field.model._meta.fields_db_projection[source_field]
+        else:
+            name = field.model_field_name + '_id'
+
+        if self._hasura_config.camel_case:
+            return humps.camelize(name)
+        return name
